@@ -1,4 +1,4 @@
-/*	$OpenBSD: imsg.c,v 1.7 2004/09/16 01:13:42 henning Exp $ */
+/*	$OpenBSD: imsg.c,v 1.9 2005/03/09 14:07:00 henning Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -83,7 +83,7 @@ imsg_get(struct imsgbuf *ibuf, struct imsg *imsg)
 
 	if (imsg->hdr.len < av) {
 		left = av - imsg->hdr.len;
-		memcpy(&ibuf->r.buf, ibuf->r.buf + imsg->hdr.len, left);
+		memmove(&ibuf->r.buf, ibuf->r.buf + imsg->hdr.len, left);
 		ibuf->r.wpos = left;
 	} else
 		ibuf->r.wpos = 0;
@@ -156,7 +156,7 @@ imsg_close(struct imsgbuf *ibuf, struct buf *msg)
 	int	n;
 
 	if ((n = buf_close(&ibuf->w, msg)) < 0) {
-			log_warnx("imsg_close: buf_add error");
+			log_warnx("imsg_close: buf_close error");
 			buf_free(msg);
 			return (-1);
 	}
