@@ -1,4 +1,4 @@
-/*	$OpenBSD: ntp.c,v 1.42 2004/11/12 17:24:52 henning Exp $ */
+/*	$OpenBSD: ntp.c,v 1.54 2005/03/08 16:33:43 henning Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -19,6 +19,7 @@
 
 #include <sys/param.h>
 #include <sys/time.h>
+#include <sys/stat.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <paths.h>
@@ -63,7 +64,7 @@ ntp_main(int pipe_prnt[2], struct ntpd_conf *nconf)
 {
 	int			 a, b, nfds, i, j, idx_peers, timeout, nullfd;
 	u_int			 pfd_elms = 0, idx2peer_elms = 0;
-	u_int			 listener_cnt, new_cnt;
+	u_int			 listener_cnt, new_cnt, sent_cnt, trial_cnt;
 	pid_t			 pid;
 	struct pollfd		*pfd = NULL;
 	struct passwd		*pw;
@@ -72,12 +73,14 @@ ntp_main(int pipe_prnt[2], struct ntpd_conf *nconf)
 	struct ntp_peer		*p;
 	struct ntp_peer		**idx2peer = NULL;
 	struct timespec		 tp;
+	struct stat		 stb;
 	time_t			 nextaction;
 	void			*newp;
 
 	switch (pid = fork()) {
 	case -1:
 		fatal("cannot fork");
+		break;
 	case 0:
 		break;
 	default:
@@ -88,11 +91,15 @@ ntp_main(int pipe_prnt[2], struct ntpd_conf *nconf)
 		fatal("getservbyname");
 
 	if ((pw = getpwnam(NTPD_USER)) == NULL)
-		fatal(NULL);
+		fatal("getpwnam");
 
 	if ((nullfd = open(_PATH_DEVNULL, O_RDWR, 0)) == -1)
 		fatal(NULL);
 
+	if (stat(pw->pw_dir, &stb) == -1)
+		fatal("stat");
+	if (stb.st_uid != 0 || (stb.st_mode & (S_IWGRP|S_IWOTH)) != 0)
+		fatal("bad privsep dir permissions");
 	if (chroot(pw->pw_dir) == -1)
 		fatal("chroot");
 	if (chdir("/") == -1)
@@ -135,9 +142,10 @@ ntp_main(int pipe_prnt[2], struct ntpd_conf *nconf)
 	conf->status.leap = LI_ALARM;
 	clock_getres(CLOCK_REALTIME, &tp);
 	b = 1000000000 / tp.tv_nsec;	/* convert to Hz */
-	for (a = 0; b > 1; a--, b >>= 1);
+	for (a = 0; b > 1; a--, b >>= 1)
+		;
 	conf->status.precision = a;
-	
+	conf->scale = 1;
 
 	log_info("ntp engine ready");
 
@@ -185,25 +193,30 @@ ntp_main(int pipe_prnt[2], struct ntpd_conf *nconf)
 		}
 
 		idx_peers = i;
+		sent_cnt = trial_cnt = 0;
 		TAILQ_FOREACH(p, &conf->ntp_peers, entry) {
 			if (p->next > 0 && p->next < nextaction)
 				nextaction = p->next;
-			if (p->next > 0 && p->next <= time(NULL))
-				client_query(p);
+			if (p->next > 0 && p->next <= time(NULL)) {
+				trial_cnt++;
+				if (client_query(p) == 0)
+					sent_cnt++;
+			}
 
 			if (p->deadline > 0 && p->deadline < nextaction)
 				nextaction = p->deadline;
 			if (p->deadline > 0 && p->deadline <= time(NULL)) {
-				log_debug("no reply from %s received in time",
-				    log_sockaddr(
-				    (struct sockaddr *)&p->addr->ss));
+				timeout = error_interval();
+				log_debug("no reply from %s received in time, "
+				    "next query %ds", log_sockaddr(
+				    (struct sockaddr *)&p->addr->ss), timeout);
 				if (p->trustlevel >= TRUSTLEVEL_BADPEER &&
 				    (p->trustlevel /= 2) < TRUSTLEVEL_BADPEER)
 					log_info("peer %s now invalid",
 					    log_sockaddr(
 					    (struct sockaddr *)&p->addr->ss));
 				client_nextaddr(p);
-				client_query(p);
+				set_next(p, timeout);
 			}
 
 			if (p->state == STATE_QUERY_SENT) {
@@ -214,6 +227,9 @@ ntp_main(int pipe_prnt[2], struct ntpd_conf *nconf)
 			}
 		}
 
+		if (trial_cnt > 0 && sent_cnt == 0 && conf->settime)
+			priv_settime(0);	/* no good peers, don't wait */
+
 		if (ibuf_main->w.queued > 0)
 			pfd[PFD_PIPE_MAIN].events |= POLLOUT;
 
@@ -233,21 +249,21 @@ ntp_main(int pipe_prnt[2], struct ntpd_conf *nconf)
 				ntp_quit = 1;
 			}
 
-		if (nfds > 0 && pfd[PFD_PIPE_MAIN].revents & POLLIN) {
+		if (nfds > 0 && pfd[PFD_PIPE_MAIN].revents & (POLLIN|POLLERR)) {
 			nfds--;
 			if (ntp_dispatch_imsg() == -1)
 				ntp_quit = 1;
 		}
 
 		for (j = 1; nfds > 0 && j < idx_peers; j++)
-			if (pfd[j].revents & POLLIN) {
+			if (pfd[j].revents & (POLLIN|POLLERR)) {
 				nfds--;
 				if (server_dispatch(pfd[j].fd, conf) == -1)
 					ntp_quit = 1;
 			}
 
 		for (; nfds > 0 && j < i; j++)
-			if (pfd[j].revents & POLLIN) {
+			if (pfd[j].revents & (POLLIN|POLLERR)) {
 				nfds--;
 				if (client_dispatch(idx2peer[j - idx_peers],
 				    conf->settime) == -1)
@@ -405,6 +421,7 @@ priv_adjtime(void)
 		conf->status.reftime = gettime();
 		conf->status.leap = LI_NOWARNING;
 		conf->status.stratum++;	/* one more than selected peer */
+		update_scale(offset_median);
 
 		if (peers[offset_cnt / 2]->addr->ss.ss_family == AF_INET)
 			conf->status.refid = ((struct sockaddr_in *)
@@ -449,3 +466,38 @@ priv_host_dns(char *name, u_int32_t peerid)
 	dlen = strlen(name) + 1;
 	imsg_compose(ibuf_main, IMSG_HOST_DNS, peerid, 0, name, dlen);
 }
+
+void
+update_scale(double offset)
+{
+	if (offset < 0)
+		offset = -offset;
+
+	if (offset > QSCALE_OFF_MAX)
+		conf->scale = 1;
+	else if (offset < QSCALE_OFF_MIN)
+		conf->scale = QSCALE_OFF_MAX / QSCALE_OFF_MIN;
+	else
+		conf->scale = QSCALE_OFF_MAX / offset;
+}
+
+time_t
+scale_interval(time_t requested)
+{
+	time_t interval, r;
+
+	interval = requested * conf->scale;
+	r = arc4random() % MAX(5, interval / 10);
+	return (interval + r);
+}
+
+time_t
+error_interval(void)
+{
+	time_t interval, r;
+
+	interval = INTERVAL_QUERY_PATHETIC * QSCALE_OFF_MAX / QSCALE_OFF_MIN;
+	r = arc4random() % (interval / 10);
+	return (interval + r);
+}
+
